import asyncio
from abc import abstractmethod

from cpl_core.configuration import ConfigurationABC
from cpl_core.dependency_injection import ServiceProviderABC
from cpl_discord.service import DiscordBotServiceABC
from discord.ext import commands

from bot_core.logging.task_logger import TaskLogger


class TaskABC(commands.Cog):
    @abstractmethod
    def __init__(self):
        commands.Cog.__init__(self)

    @ServiceProviderABC.inject
    async def _wait_until_ready(self, config: ConfigurationABC, logger: TaskLogger, bot: DiscordBotServiceABC):
        logger.debug(__name__, f"Waiting before checking steam special offers")
        await bot.wait_until_ready()

        async def wait():
            is_ready = config.get_configuration("IS_READY")
            if is_ready != "true":
                await asyncio.sleep(1)
                await wait()

        await wait()
