ALTER TABLE `Clients`
    CHANGE `CreatedAt` `CreatedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6);

ALTER TABLE `Clients`
    CHANGE `LastModifiedAt` `LastModifiedAt` DATETIME(6) NULL DEFAULT CURRENT_TIMESTAMP(6) ON UPDATE CURRENT_TIMESTAMP(6);

CREATE TABLE IF NOT EXISTS `ClientsHistory`
(
    `Id`              BIGINT(20)  NOT NULL,
    `DiscordId`       BIGINT(20)  NOT NULL,
    `SentMessageCount`      BIGINT(20)  NOT NULL DEFAULT 0,
    `ReceivedMessageCount`  BIGINT(20)  NOT NULL DEFAULT 0,
    `DeletedMessageCount`   BIGINT(20)  NOT NULL DEFAULT 0,
    `ReceivedCommandsCount` BIGINT(20)  NOT NULL DEFAULT 0,
    `MovedUsersCount`       BIGINT(20)  NOT NULL DEFAULT 0,
    `ServerId`              BIGINT(20)           DEFAULT NULL,
    `Deleted`               BOOL                 DEFAULT FALSE,
    `DateFrom`              DATETIME(6) NOT NULL,
    `DateTo`                DATETIME(6) NOT NULL
);

DROP TRIGGER IF EXISTS `TR_ClientsUpdate`;

CREATE TRIGGER `TR_ClientsUpdate`
    AFTER UPDATE
    ON `Clients`
    FOR EACH ROW
BEGIN
    INSERT INTO `ClientsHistory` (
        `Id`, `DiscordId`, `SentMessageCount`, `ReceivedMessageCount`, `DeletedMessageCount`,
        `ReceivedCommandsCount`, `MovedUsersCount`, `ServerId`, `DateFrom`, `DateTo`
    )
    VALUES (
        OLD.ClientId, OLD.DiscordClientId, OLD.SentMessageCount, OLD.ReceivedMessageCount, OLD.DeletedMessageCount,
        OLD.ReceivedCommandsCount, OLD.MovedUsersCount, OLD.ServerId, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6)
    );
END;

DROP TRIGGER IF EXISTS `TR_ClientsDelete`;

CREATE TRIGGER `TR_ClientsDelete`
    AFTER DELETE
    ON `Clients`
    FOR EACH ROW
BEGIN
    INSERT INTO `ClientsHistory` (
        `Id`, `DiscordId`, `SentMessageCount`, `ReceivedMessageCount`, `DeletedMessageCount`,
        `ReceivedCommandsCount`, `MovedUsersCount`, `ServerId`, `Deleted`, `DateFrom`, `DateTo`
    )
    VALUES (
        OLD.ClientId, OLD.DiscordClientId, OLD.SentMessageCount, OLD.ReceivedMessageCount, OLD.DeletedMessageCount,
        OLD.ReceivedCommandsCount, OLD.MovedUsersCount, OLD.ServerId, TRUE, OLD.LastModifiedAt, CURRENT_TIMESTAMP(6)
    );
END;