from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.data_seeder_abc import DataSeederABC
from bot_data.abc.technician_config_repository_abc import TechnicianConfigRepositoryABC
from bot_data.model.technician_config import TechnicianConfig
from bot_data.model.technician_id_config import TechnicianIdConfig
from bot_data.model.technician_ping_url_config import TechnicianPingUrlConfig


class TechnicianConfigSeeder(DataSeederABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        technician_config: TechnicianConfigRepositoryABC,
        db: DatabaseContextABC,
    ):
        DataSeederABC.__init__(self)

        self._logger = logger
        self._technician_config = technician_config
        self._db = db

    async def seed(self):
        try:
            if self._technician_config.does_technician_config_exists():
                return

            config = TechnicianConfig(
                "https://git.sh-edraft.de/sh-edraft.de/kd_discord_bot/wiki/Befehle",
                8,
                8,
                1000000,
                {},
                List(int, [240160344557879316]),
                List(str, ["www.google.com", "www.sh-edraft.de", "www.keksdose-gaming.de"]),
            )

            self._technician_config.add_technician_config(config)
            for technician in config.technician_ids:
                self._technician_config.add_technician_id_config(TechnicianIdConfig(technician))

            for url in config.ping_urls:
                self._technician_config.add_technician_ping_url_config(TechnicianPingUrlConfig(url))

            self._db.save_changes()
            self._logger.debug(__name__, "Seeded technician config")
        except Exception as e:
            self._logger.error(__name__, f"Seeding technician config failed", e)
